# Copyright 2009 Canonical Ltd.
#
# This file is part of desktopcouch.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
"Desktop Couch helper files"

from __future__ import with_statement
import os
import re
import errno
import time

def find_pid():
    # Work out whether CouchDB is running by looking at its pid file
    from desktopcouch import local_files
    fp = open(local_files.FILE_PID)
    pid = int(fp.read())
    fp.close()
    try:
        os.kill(pid, 0)
    except os.error, detail:
        if detail.errno == errno.ESRCH:
            # pidfile is stale
            # start CouchDB by running the startup script
            print "Desktop CouchDB is not running; starting it."
            from desktopcouch import start_local_couchdb
            start_local_couchdb.start_couchdb()
            time.sleep(2) # give the process a chance to start

    # get the pid
    try:
        with open(local_files.FILE_PID) as pid_file:
            pid = int(pid_file.read().strip())
    except IOError, e:
        if e.errno == ENOENT:
            raise RuntimeError("desktop-couch not started")
        else:
            raise
    return pid

def find_port(pid):
    # Look in the CouchDB log to find the port number, someday.
    # Currently, we have to grovel around in /proc instead.
    # Oh, the huge manatee... (this replaced an lsof shell recipe
    # which was shorter but less reliable)

    proc_dir = "/proc/%s" % (pid,)

    # check to make sure that the process still exists
    if not os.path.isdir(proc_dir):
        raise RuntimeError("Stale PID file or desktop-couch crashed")

    # check to make sure it is actually a desktop-couch instance
    with open(os.path.join(proc_dir, 'cmdline')) as cmd_file:
        cmd = cmd_file.read()
    if re.search('desktop-couch', cmd) is None:
        raise RuntimeError("Stale PID file")

    # enumerate the process' file descriptors
    fd_dir = os.path.join(proc_dir, 'fd')
    fd_paths = [os.readlink(os.path.join(fd_dir, fd))
                for fd in os.listdir(fd_dir)]

    # identify socket fds
    socket_matches = [re.match('socket:\\[([0-9]+)\\]', p) for p in fd_paths]
    # extract their inode numbers
    socket_inodes = [m.group(1) for m in socket_matches if m is not None]

    # construct a subexpression which matches any one of these inodes
    inode_subexp = "|".join(socket_inodes)
    # construct regexp to match /proc/net/tcp entries which are listening
    # sockets having one of the given inode numbers
    listening_regexp = re.compile(r'''
        \s*\d+:\s*                # sl
        0100007F:([0-9A-F]{4})\s+ # local_address
        00000000:0000\s+          # rem_address
        0A\s+                     # st (0A = listening)
        [0-9A-F]{8}:              # tx_queue
        [0-9A-F]{8}\s+            # rx_queue
        [0-9A-F]{2}:              # tr
        [0-9A-F]{8}\s+            # tm->when
        [0-9A-F]{8}\s*            # retrnsmt
        \d+\s+\d+\s+              # uid, timeout
        (?:%s)\s+                 # inode
        ''' % (inode_subexp,), re.VERBOSE)

    # extract the TCP port from the first matching line in /proc/$pid/net/tcp
    port = None
    with open(os.path.join(proc_dir, 'net', 'tcp')) as tcp_file:
        for line in tcp_file:
            match = listening_regexp.match(line)
            if match is not None:
                port = int(match.group(1), 16)
                break
    if port is None:
        raise RuntimeError("Unable to find listening port")

    return str(port)
