#!/usr/bin/python
# Copyright 2009 Canonical Ltd.
#
# This file is part of desktopcouch.
#
#  desktopcouch is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3
# as published by the Free Software Foundation.
#
# desktopcouch is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with desktopcouch.  If not, see <http://www.gnu.org/licenses/>.
#
# Author: Stuart Langridge <stuart.langridge@canonical.com>
"""Specify location of files relevant to desktop CouchDB.

Checks to see whether we're running out of the source tree or not.
"""

import os
import xdg.BaseDirectory
import subprocess
import sys

def mkpath(rootdir, path):
    "Remove .. from paths"
    return os.path.realpath(os.path.join(rootdir, path))

if os.path.isdir(os.path.join(
  os.path.split(__file__)[0],"..", "..", "..", "sourcecode")):
    IN_SOURCE_TREE = True
else:
    IN_SOURCE_TREE = False

if IN_SOURCE_TREE:
    rootdir = os.path.join(os.path.split(__file__)[0], "..", "..", "..", "tmp")
    FILE_INI = mkpath(rootdir, "desktop-couchdb.ini")
    DIR_DB = mkpath(rootdir, "desktop-couch-files")
    if not os.path.isdir(DIR_DB): 
        os.makedirs(DIR_DB)
else:
    rootdir = os.path.join(xdg.BaseDirectory.xdg_cache_home, "desktop-couch")
    if not os.path.isdir(rootdir):
        os.makedirs(rootdir)
    config_dir = xdg.BaseDirectory.save_config_path("desktop-couch")
    FILE_INI = os.path.join(config_dir, "desktop-couchdb.ini")
    DIR_DB = xdg.BaseDirectory.save_data_path("desktop-couch")
    
FILE_PID = mkpath(rootdir, "desktop-couchdb.pid")
FILE_LOG = mkpath(rootdir, "desktop-couchdb.log")
FILE_STDOUT = mkpath(rootdir, "desktop-couchdb.stdout")
FILE_STDERR = mkpath(rootdir, "desktop-couchdb.stderr")


COUCH_EXE = os.environ.get('COUCHDB')
if not COUCH_EXE:
    for x in os.environ['PATH'].split(':'):
        if os.path.exists(os.path.join(x, 'couchdb')):
            COUCH_EXE = os.path.join(x, 'couchdb')
if not COUCH_EXE:
    raise ImportError("Could not find couchdb")

def couch_chain_flag():
    process = subprocess.Popen([COUCH_EXE, '-V'], shell=False,
        stdout=subprocess.PIPE)
    line = process.stdout.read().split('\n')[0]
    couchversion = line.split()[-1]
    if couchversion.startswith('0.1'):
        chain = '-a'
    else:
        chain = '-C'

    return chain


# You will need to add -b or -k on the end of this
COUCH_EXEC_COMMAND = [COUCH_EXE, couch_chain_flag(), FILE_INI, '-p', FILE_PID,
  '-o', FILE_STDOUT, '-e', FILE_STDERR]

